@@ -49,7 +49,7 @@ greeter_langauge_initialize_model (void)
 
   gtk_list_store_append (lang_model, &iter);
   gtk_list_store_set (lang_model, &iter,
-		      TRANSLATED_NAME_COLUMN, _("System default"),
+		      TRANSLATED_NAME_COLUMN, _("System Default"),
 		      UNTRANSLATED_NAME_COLUMN, NULL,
 		      LOCALE_COLUMN, DEFAULT_LANGUAGE,
 		      -1);
@@ -108,12 +108,6 @@ greeter_language_get_language (const char *old_language)
 {
   gchar *retval = NULL;
 
-#if 0
-  if (greeter_current_user == NULL)
-    greeter_abort ("greeter_language_get_language: curuser==NULL. Mail <mkp@mkp.net> with " \
-		    "information on your PAM and user database setup");
-#endif
-  
   /* Don't save language unless told otherwise */
   savelang = FALSE;
 
@@ -135,7 +129,7 @@ greeter_language_get_language (const char *old_language)
 	  char *current_name, *saved_name;
 
 	  if (strcmp (current_language, DEFAULT_LANGUAGE) == 0)
-	    current_name = g_strdup (_("System default"));
+	    current_name = g_strdup (_("System Default"));
 	  else
 	    current_name = gdm_lang_name (current_language,
 					  FALSE /* never_encoding */,
@@ -143,7 +137,7 @@ greeter_language_get_language (const char *old_language)
 					  TRUE /* untranslated */,
 					  TRUE /* markup */);
 	  if (strcmp (old_language, "") == 0)
-	    saved_name = g_strdup (_("System default"));
+	    saved_name = g_strdup (_("System Default"));
 	  else
 	    saved_name = gdm_lang_name (old_language,
 					FALSE /* never_encoding */,
@@ -157,7 +151,7 @@ greeter_language_get_language (const char *old_language)
 	  g_free (current_name);
 	  g_free (saved_name);
 
-	  savelang = greeter_query (msg);
+	  savelang = greeter_query (msg, _("Make _Default"), _("Just For _This Session"));
 	  g_free (msg);
 	}
     }
@@ -202,6 +196,10 @@ greeter_action_language (GreeterItemInfo *info,
 					    GTK_STOCK_OK,
 					    GTK_RESPONSE_OK,
 					    NULL);
+      gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+
+      gtk_dialog_set_default_response (GTK_DIALOG (dialog),
+				       GTK_RESPONSE_OK);
       g_object_add_weak_pointer (G_OBJECT (dialog), (gpointer *) &dialog);
       s = g_strdup_printf ("<span size=\"x-large\" weight=\"bold\">%s</span>",
 			   _("Select a language for your session to use:"));
@@ -211,6 +209,7 @@ greeter_action_language (GreeterItemInfo *info,
       gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
 			  label, FALSE, FALSE, 0);
       view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (lang_model));
+      gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (view), TRUE);
       /* FIXME: we should handle this better, but things really look
        * like crap if we aren't always LTR */
       gtk_widget_set_direction (view, GTK_TEXT_DIR_LTR);
